"""Look up abbreviations for terms (or the reverse).

Usage:
  abbr [options] <term>
  abbr [options] -r <abbreviation>
  abbr (-h | --help)
  abbr --version

Options:
  --version           Show version.
  -h --help           Show this screen.
  -r --reverse        Reverse the look up. Find terms for an abbreviation.
  -n --limit <n>      Limit the number of result [default: 25].
  -m --min-stars <m>  Include only items with number of stars equal or above <m>.
                      Allowed values are 0-5 (inclusive) [default: 0].
  -w --only-words     List only the words (terms or abbreviations) without category
                      and rating. Helpful when used in a bash script.

"""

import sys

from docopt import docopt

import abbr
from abbr import core
from abbr.exitstatus import ExitStatus


def main():
    try:
        exit_status = core.main(docopt(__doc__, version=f'abbr {abbr.__version__}'))
    except KeyboardInterrupt:
        exit_status = ExitStatus.ERROR_CTRL_C

    sys.exit(exit_status.value)


if __name__ == '__main__':
    main()



import requests


class BackendClient(object):

    def __init__(self, query: str, _reversed: bool = False, order=0):
        self.query = query
        self.reversed = _reversed
        self.order = order

    def execute(self) -> tuple[int, str]:
        pass


class AbbreviationsClient(BackendClient):
    base_url = "https://www.abbreviations.com"

    def __init__(self, query: str, _reversed: bool = False, order=0):
        super().__init__(query, _reversed, order)

    def execute(self):
        query_type = 1 if self.reversed else 2  # allowed valuae are 1 (abbr->term) or 2 (term->abbr)
        order = self.order  # allowed values are 0 (popularity), 1 (alphabetically), and 2 (category)
        page = 99999  # page number; special number 99999 means retrive all pages
        params = {'st': self.query, 'o': order, 'qtype': query_type, 'p': page}
        response = requests.get(f"{self.base_url}/serp.php", params=params, allow_redirects=False)
        return response.status_code, response.text



import sys

from lxml import etree


class BaseScraper(object):

    def __init__(self, html: str, limit: int, min_stars: int, reversed_flag: bool):
        super().__init__()
        self.html = html
        self.limit = limit if limit > 0 else sys.maxsize
        self.min_stars = min_stars
        self.reversed = reversed_flag
        self._words: list[str] = list()
        self._words_stars: dict[int] = dict()
        self._words_categories: dict[set] = dict()
        self.parse()

    def parse(self):
        pass

    # noinspection PyMethodMayBeStatic
    def is_valid_word(self, word: str) -> bool:
        return len(word) > 1

    def normalize(self, word):
        if self.reversed:
            normalized = word.title()
        else:
            normalized = word.lower()
        return normalized

    def add_word(self, word: str):
        word = self.normalize(word)
        if word not in self._words:
            self._words.append(word)

    def add_word_stars(self, word: str, star_count: int):
        word = self.normalize(word)
        current_value = self._words_stars.get(word, 0)
        if current_value == 0:
            self._words_stars[word] = star_count
        elif star_count == 0:
            self._words_stars[word] = current_value
        else:
            self._words_stars[word] = round((current_value + star_count) / 2)

    def add_word_category(self, word: str, category: str):
        word = self.normalize(word)
        self._words_categories[word] = self._words_categories.get(word, set()) | {category}

    def words(self) -> list[str]:
        return self._words

    def words_stars(self) -> dict[int]:
        return self._words_stars

    def words_categories(self) -> dict[set]:
        return self._words_categories


class XPathScraper(BaseScraper):

    def __init__(self, html: str, limit: int = 0, min_stars=0, reversed_flag: bool = False):
        super().__init__(html, limit, min_stars, reversed_flag)

    def parse(self):
        elements = \
            etree.HTML(self.html).xpath(
                "//table[contains(@class,'no-margin')]/tbody/tr")  # type: list[etree._Element]

        if not elements:
            return

        total = 0
        for e in elements:
            star_count = int(e.xpath("count(.//span[contains(@class,'sf')])"))
            if star_count < self.min_stars:
                continue

            if self.reversed:
                word = e.xpath(".//p[contains(@class,'desc')]")[0].text
            else:
                word = e.xpath("./td[1]/a[1]")[0].text

            if not self.is_valid_word(word):
                continue

            self.add_word(word)
            self.add_word_stars(word, star_count)

            category = e.xpath(".//p[contains(@class,'path')]/a[1]")[0].text
            self.add_word_category(word, category)

            total += 1
            if total >= self.limit:
                break



import sys

import colorful as cf
from requests.exceptions import ConnectionError, ConnectTimeout

from abbr.clients import AbbreviationsClient
from abbr.exitstatus import ExitStatus
from abbr.scrapers import XPathScraper

_repo_url = 'https://github.com/mhadidg/abbr-cli'

_color_palette = {
    'red': '#EF5350',
    'abbr': '#FFFFFF',
    'rating': '#FFFFFF',
    'star': '#757575',
    'starBraket': '#757575',
    'category': '#757575',
    'categorySeparator': '#757575',
}


# noinspection PyBroadException
def main(args) -> ExitStatus:
    cf.use_palette(_color_palette)

    term = args['<term>']
    abbrv = args['<abbreviation>']
    reversed_flag = args['--reverse']
    min_stars = int(args['--min-stars'])
    fancy_output = not args['--only-words']
    limit = int(args['--limit'])

    query = abbrv if reversed_flag else term

    try:
        status_code, html = AbbreviationsClient(query, reversed_flag).execute()
    except ConnectTimeout:
        eprint(f"Connection timed out!")
        return ExitStatus.ERROR
    except ConnectionError:
        eprint(f"Connection failed! Make sure you're connected to the internet.")
        return ExitStatus.ERROR
    except Exception:
        eprint(f"Unexpected error! Please report the issue on {_repo_url}.")
        return ExitStatus.ERROR

    if status_code == 302:
        if reversed_flag and fancy_output:
            print("Zero terms. Is {} a term? Then don't use -r flag.".format(cf.bold_red(abbrv)))
        elif fancy_output:
            print("Zero abbreviations. Is {} an abbreviation? Then use with -r flag.".format(cf.bold_red(term)))
        return ExitStatus.SUCCESS

    try:
        scraper = XPathScraper(html, limit, min_stars, reversed_flag)
    except Exception:
        eprint(f"Unexpected Error! Please report the issue on {_repo_url}.")
        return ExitStatus.ERROR

    # The 'words' here could be either abbreivations or terms
    # depending on whether or not the --reverse flag is present.
    words = scraper.words()

    if not words:
        if reversed_flag and fancy_output:
            print("Zero terms.")
        elif fancy_output:
            print("Zero abbreviations.")
        return ExitStatus.SUCCESS

    if fancy_output:
        words_stars = scraper.words_stars()
        words_categories = scraper.words_categories()
        simplified_fancy_print(words, words_categories, words_stars)
    else:
        simple_print(words)

    return ExitStatus.SUCCESS


def fancy_print(words: list[str], category_dict: dict[set], star_dict: dict[int]):
    for abbr in words:
        star_count = star_dict[abbr]
        category = ', '.join(category_dict[abbr])
        filling_spaces = (' ' * (5 - star_count))
        print(
            cf.starBraket(" [{}] ".format(cf.star('×' * star_count) + filling_spaces))
            + cf.abbr(abbr)
            + cf.categorySeparator(' ~ ')
            + cf.category(category))


def simplified_fancy_print(words: list[str], category_dict: dict[set], star_dict: dict[int]):
    for abbr in words:
        star_count = star_dict[abbr]
        category = ', '.join(category_dict[abbr])
        print(
            "({}) ".format('{}/5'.format('-' if star_count == 0 else star_count))
            + abbr
            + cf.categorySeparator(' ~ ')
            + cf.category(category))


def simple_print(words: list[str]):
    for abbr in words:
        print(abbr)


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)



#!/usr/bin/env python
#
# Adpated from httpie project (https://github.com/httpie/httpie)

import codecs
import os
import sys
from distutils.dir_util import remove_tree

from setuptools import setup, find_packages

import abbr

# 'setup.py publish' shortcut.
if sys.argv[-1] == 'publish':
    os.system('./setup.py clean --all')
    os.system('./setup.py sdist bdist_wheel')
    os.system('twine upload dist/*')
    sys.exit()
# 'setup.py clean-all' shortcut.
elif sys.argv[-1] == 'clean-all':
    os.system('./setup.py clean --all')
    remove_tree('./dist')
    remove_tree('./abbr_cli.egg-info')
    sys.exit()

tests_require = []

dev_require = [
    *tests_require,
    'twine',
    'wheel',
]

install_requires = [
    'requests>=2.22.0',
    'lxml>=4.5.0',
    'docopt>=0.6.2',
    'colorful>=0.5.4',
    'setuptools',
]

install_requires_win_only = [
    'colorama>=0.2.4',
]

# sdist
if 'bdist_wheel' not in sys.argv:
    if 'win32' in str(sys.platform).lower():
        # Terminal colors for Windows
        install_requires.extend(install_requires_win_only)

# bdist_wheel
extras_require = {
    'dev': dev_require,
    'test': tests_require,
    # https://wheel.readthedocs.io/en/latest/#defining-conditional-dependencies
    ':sys_platform == "win32"': install_requires_win_only,
}


def long_description():
    with codecs.open('README.md', 'r', encoding='utf8') as file:
        return file.read()


setup(
    name=abbr.__project_name__,
    version=abbr.__version__,
    description=abbr.__description__,
    long_description=long_description(),
    long_description_content_type='text/markdown',
    author=abbr.__author__,
    author_email=abbr.__author_email__,
    license=abbr.__licence__,
    package_dir={'abbr': 'abbr'},
    packages=find_packages(include=['abbr']),
    entry_points={
        'console_scripts': [
            'abbr = abbr.__main__:main',
        ],
    },
    python_requires='>=3.6',
    extras_require=extras_require,
    install_requires=install_requires,
    classifiers=[
        'Environment :: Console',
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3 :: Only',
        'Intended Audience :: Education',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'Topic :: Terminals',
        'Topic :: Utilities'
    ],
    project_urls={
        'GitHub': 'https://github.com/mhadidg/abbr-cli',
    },
)
